@@ -1,6 +1,6 @@
 
 /*
- * $Id: client_side.c,v 1.88 1997/02/07 04:57:10 wessels Exp $
+ * $Id: client_side.c,v 1.90 1997/02/24 20:21:37 wessels Exp $
  *
  * DEBUG: section 33    Client-side Routines
  * AUTHOR: Duane Wessels
@@ -159,6 +159,8 @@ clientAccessCheck(icpStateData * icpState, void (*handler) (icpStateData *, int)
 	    icpState->ident.ident,
 	    ICP_IDENT_SZ);
     }
+    /* This so we can have SRC ACLs for cache_host_acl. */
+    icpState->request->client_addr = icpState->peer.sin_addr;
 #if USE_PROXY_AUTH
     if (clientProxyAuthCheck(icpState) == 0) {
 	char *wbuf = NULL;
@@ -430,6 +432,7 @@ icpProcessExpired(int fd, void *data)
 	icpState->method);
     /* NOTE, don't call storeLockObject(), storeCreateEntry() does it */
     storeClientListAdd(entry, fd, 0);
+    storeClientListAdd(icpState->old_entry, fd, 0);
 
     entry->lastmod = icpState->old_entry->lastmod;
     debug(33, 5, "icpProcessExpired: setting lmt = %d\n",
