@@ -1,6 +1,6 @@
 
 /*
- * $Id: squid.h,v 1.93 1997/01/31 22:30:36 wessels Exp $
+ * $Id: squid.h,v 1.95 1997/02/23 09:02:21 wessels Exp $
  *
  * AUTHOR: Duane Wessels
  *
@@ -45,11 +45,17 @@
  * we get in trouble if we try to increase it.  barf.
  */
 
-#ifndef _SQUID_LINUX_
+#ifdef _SQUID_LINUX_
+	/* Cannot increase FD_SETSIZE on Linux */
+#elif _SQUID_FREEBSD_
+	/* Cannot increase FD_SETSIZE on FreeBSD */
+	/* Marian Durkovic <marian@svf.stuba.sk> */
+#else
+	/* Increase FD_SETSIZE if SQUID_MAXFD is bigger */
 #if SQUID_MAXFD > FD_SETSIZE
 #define FD_SETSIZE SQUID_MAXFD
 #endif
-#endif /* _SQUID_LINUX_ */
+#endif /* */
 
 #if HAVE_UNISTD_H
 #include <unistd.h>
@@ -354,6 +360,8 @@ extern int passStart _PARAMS((int fd,
 	int *size_ptr));
 extern void identStart _PARAMS((int, icpStateData *,
 	void       (*callback) _PARAMS((void *))));
+extern int httpAnonAllowed _PARAMS((const char *line));
+extern int httpAnonDenied _PARAMS((const char *line));
 
 extern const char *const dash_str;
 extern const char *const null_string;
